//
// file : aiController.h
//

#ifndef _OCTABALL_NPC_CONTROLLER_H_
#define _OCTABALL_NPC_CONTROLLER_H_

#include "ai_fsm/controller.h"
#include "ai_fsm/machine.h" 
#include "aiState.h" // the custom states


class cNPC_controller : public cAI_controller
{
public:
	cAI_machine *m_machine;
	cNPC_controller()
	{
		// construct the state machine 
		m_machine = new cAI_machine(FSM_MACH_NPC, this);
		
		// add the custom states to the machine and set default
	
		cAI_state_idle *idleState = new cAI_state_idle(this);
		m_machine->addState(idleState);
		m_machine->addState(new cAI_state_reset_run(this));
		m_machine->addState(new cAI_state_search_pipe(this));
		m_machine->addState(new cAI_state_approach(this));
		m_machine->addState(new cAI_state_approach_character(this));
		m_machine->addState(new cAI_state_approach_bumper(this));
		m_machine->addState(new cAI_state_approach_pipe_breakage(this));
		m_machine->addState(new cAI_state_drive_pin(this));
		m_machine->addState(new cAI_state_attack_bumper(this));
		m_machine->addState(new cAI_state_attack_pattern_alpha(this));
		m_machine->addState(new cAI_state_attack_pattern_beta(this));
		m_machine->addState(new cAI_state_attack_pattern_delta(this));

		m_machine->addState(new cAI_state_predictApproaching(this));
		m_machine->addState(new cAI_state_predictRetreating(this));
		m_machine->setDefaultState(idleState);
	}

	void update(float t)
	{
		if(ready == false)//m_theActor)
		{
			m_machine->reset();
			ready = true;
			return;
		}
		updatePerceptions(t); // yup...changing it up?
		m_machine->update(t); // update current state
	}

	KeyFrame getNextKey()
	{
		KeyFrame key = { targetTime, targetx, targety, 0 };
		return key;
		
	}

	void updatePerceptions(float t);
	void initialize();

	bool ready = false;
	bool bFoundPipe = false;
	bool bDropPin = false;

	int fireThisFrame = 0;  // 0:none 1:left 2:right

	// world awareness
	int posx;
	int posy;
	float targetTime = 1.f;
	int targetx = 400;
	int targety = 100;

	int searchElevation; // lowest found
	int searchPos;       // best found
	int playerPos;
	int bumper01Pos;
	int bumper02Pos;
	int bumper03Pos;
	int pipePos;

	//cMesh_obj   *m_theActor; // this objects graphical representation
	
	// perception data
	//cOctaGrid *m_theGrid;
	//cMesh_obj *m_theBall;
	//cMesh_obj *m_thePlayer;

	//bool m_bBallDirectionChanged; // main trigger

	//cVector2 m_vecKnownBallTrajectory;
	//cVector2 m_vecKnownBallCollisionPos;

	// internals
	//float m_safetyRadius; // no soup for you.
	
};


#endif//_OCTABALL_NPC_CONTROLLER_H_


