//
// file : aiState.h
//

#ifndef _AI_STATE_H_
#define _AI_STATE_H_

#include "ai_fsm/state_base.h"

struct KeyFrame
{
	//KeyFrame(float time, float posX, float posY, int trigger) 
	//	: t(time), px(posX), py(posY), aux(trigger) {}
	float t;  // finish time
	float px; // position
	float py;
	int aux;  // trigger on end frame (0=none)
};

//
// idle 
// resetRun
// searchPipe
// approachCharacter
// approachBumperObj
// approachPipeFound
// drivePuncturePin
//
// disruptBumperObj
// attackPatternAlpha
// attackPatternBeta
// attackPatternCornholio
//

// ------------------------------------------------------------------------------
class cAI_state_idle : public cAI_state
{
public:
	cAI_state_idle(cAI_controller *parent) : cAI_state(FSM_STATE_IDLE, parent) {}
	void update(float t);
	eAI_state checkTransition(float t);
	void exit();
};


class cAI_state_reset_run : public cAI_state
{
public:
	cAI_state_reset_run(cAI_controller *parent) : cAI_state(FSM_STATE_RESET, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};


class cAI_state_search_pipe : public cAI_state
{
public:
	cAI_state_search_pipe(cAI_controller *parent) : cAI_state(FSM_STATE_SEARCH, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();


	float y = 100.f;
	
	KeyFrame keys[8] = { {1.f, 350.f, y, 0}, {0.5f, 400.f, y, 2}, {0.5f, 450.f, y, 0}, {0.5f, 500.f, y, 1}, {0.5f, 550.f, y, 0}, {0.5f, 600.f, y, 2}, {0.5f, 650.f, y, 0}, {1.f, 700.f, y, 1} };
	const int numKeys = 8;
	int keyCursor = 0;

	int bestProbe = -1;
};


class cAI_state_approach : public cAI_state
{
public:
	cAI_state_approach(cAI_controller *parent) : cAI_state(FSM_STATE_APPROACH, parent) {}
	void enter();
	void update(float t);
	eAI_state  checkTransition(float t);                                     // using this as enter stage right
	void exit();

	bool  m_bPositionReached; // state change trigger
	float m_fElapsed;         // fail safe escape trigger;
};


class cAI_state_approach_character : public cAI_state
{
public:
	cAI_state_approach_character(cAI_controller *parent) : cAI_state(FSM_STATE_APPROACH_CHARACTER, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};


class cAI_state_approach_bumper : public cAI_state
{
public:
	cAI_state_approach_bumper(cAI_controller *parent) : cAI_state(FSM_STATE_APPROACH_BUMPER, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};


class cAI_state_approach_pipe_breakage : public cAI_state
{
public:
	cAI_state_approach_pipe_breakage(cAI_controller *parent) : cAI_state(FSM_STATE_APPROACH_PIPE, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};


// drivePuncturePin
class cAI_state_drive_pin : public cAI_state
{
public:
	cAI_state_drive_pin(cAI_controller *parent) : cAI_state(FSM_STATE_DRIVE_PIN, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};



// attackBumper
class cAI_state_attack_bumper : public cAI_state
{
public:
	cAI_state_attack_bumper(cAI_controller *parent) : cAI_state(FSM_STATE_ATTACK_BUMPER, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};


// attackPatternAlpha
class cAI_state_attack_pattern_alpha : public cAI_state
{
public:
	cAI_state_attack_pattern_alpha(cAI_controller *parent) : cAI_state(FSM_STATE_ATTACK_PATTERN_ALPHA, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();

	bool m_bPositionReached = false;

	float y = 40.f;
	float x = 0.f;  // runtime
	KeyFrame keys[5] = { {1.f, x, y, 0}, {0.15f, x+30.f, y+30.f, 2}, {0.33f, x-30.f, y+60.f, 1}, {0.33f, x+30.f, y+90.f, 2}, {0.33f, x-30.f, y+120.f, 1} };
	const int numKeys = 5;
	int keyCursor = 0;
};


// attackPatternBeta
class cAI_state_attack_pattern_beta : public cAI_state
{
public:
	cAI_state_attack_pattern_beta(cAI_controller *parent) : cAI_state(FSM_STATE_ATTACK_PATTERN_BETA, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};

// attackPatternDelta
class cAI_state_attack_pattern_delta : public cAI_state
{
public:
	cAI_state_attack_pattern_delta(cAI_controller *parent) : cAI_state(FSM_STATE_ATTACK_PATTERN_DELTA, parent) {}
	void update(float dt);
	eAI_state checkTransition(float t);
	void exit();
};



// ------------------------------------------------------------------------------
class cAI_state_predictApproaching : public cAI_state
{
public:
	cAI_state_predictApproaching(cAI_controller *parent) : cAI_state(FSM_STATE_PREDICT_APPROACHING, parent) {}
	void enter();
	void update(float t);
	eAI_state checkTransition(float t);
	void exit();

	bool m_bPositionNotReachable;
};


// ------------------------------------------------------------------------------
class cAI_state_predictRetreating : public cAI_state
{
public:
	cAI_state_predictRetreating(cAI_controller *parent) : cAI_state(FSM_STATE_PREDICT_RETREATING, parent) {}
	void update(float t);
	eAI_state checkTransition(float t);
	void exit();
};


#endif//_AI_STATE_H_

