//
// file : aiMachine.cpp
//

#include "machine.h"

void cAI_machine::update(float t)
{
	// don't do anything if there are no states
	if(m_stateList.size() == 0)
		return;
	// don't do anything if there is no current state and no default state
	if(!m_currentState)
		m_currentState = m_defaultState;
	if(!m_currentState)
		return;

	// update the current state and check for a transition
	eAI_state oldStateId = m_currentState->m_type;
	m_goalId = m_currentState->checkTransition(t);

	// switch if there was a transition
	if(m_goalId != oldStateId)
	{
		if(doTransitionState(m_goalId))
		{
			m_currentState->exit();
			m_currentState = m_goalState;
			m_currentState->enter();
		}
	}
	m_currentState->update(t);
}


void cAI_machine::reset()
{
	exit();
	if(m_currentState)
		m_currentState->exit();
	m_currentState = m_defaultState;

	// initialize all the states
	for(unsigned int i=0; i<m_stateList.size(); i++)
		m_stateList[i]->initialize();
	// and enter the m_defaultState if any
	if(m_currentState)
		m_currentState->enter();
}


bool cAI_machine::doTransitionState(eAI_state goal)
{
	// don't do anything if there are no states
	if(m_stateList.size() == 0)
		return false;
	// determine if there is state of type "goal" in the list and switch to it.
	for(unsigned int i=0; i<m_stateList.size(); i++)
	{
		if(m_stateList[i]->m_type == goal)
		{
			m_goalState = m_stateList[i];
			return true;
		}
	}
	return false;
}

