#pragma once
#include "../SFX/Lightning.h"


class Bumper : olc::PGEX
{
public:
	Bumper(int x, int y) { p.x = x; p.y = y; }

	void init()
	{
		lightning = LightningBolt(vec2(p.x+o1.x, p.y+o1.y), vec2(p.x+o2.x, p.y+o2.y), olc::WHITE);
		lightning.CreateBolt(vec2(o1.x, o1.y), vec2(o2.x, o2.y), &lightning.segments1);
		lightning.CreateBolt(vec2(o1.x, o1.y), vec2(o2.x, o2.y), &lightning.segments2);
	}

	void update(float dt, vec2 cam)
	{
		auto IsPointInCircle = [](float x1, float y1, float r1, float px, float py)
		{
			return fabs((x1 - px)*(x1 - px) + (y1 - py)*(y1 - py)) < (r1 * r1);
		};

		vec2 mp = {(float)pge->GetMouseX(), (float)pge->GetMouseY()};
		mp += cam; // convert view to world space

#pragma region picking
		//  picking -----------------------------------
		if (pge->GetMouse(0).bPressed)
		{
			if (IsPointInCircle(p.x, p.y, r, mp.x, mp.y))                 selectedID = 0; // base thumb
			else if (IsPointInCircle(p.x+o1.x, p.y+o1.y, r1, mp.x, mp.y)) selectedID = 1; // thumb 1
			else if (IsPointInCircle(p.x+o2.x, p.y+o2.y, r2, mp.x, mp.y)) selectedID = 2; // thumb 2
			else selectedID = -1;			
		}
		if (pge->GetMouse(0).bReleased)
		{
			if (selectedID == -1)
			{
				// do other unrelated stuff here
				// this is not a pick click
			}
			selectedID = -1;
		} // dragging ----------------------------------
		if (pge->GetMouse(0).bHeld)
		{
			switch (selectedID)
			{
			case 0: p = mp; break;
			case 1: o1 = mp - p; break;
			case 2: o2 = mp - p; break;
			}
		}
#pragma endregion

		if (abs(o1.x) < r && abs(o1.y) < r) o1.x--;
		if (abs(o2.x) < r && abs(o2.y) < r) o2.x++;
		lightning.Update(dt, p+o1-cam, p+o2-cam);
		t += dt;

		normal = (o1 - o2).perp().norm(); // global normal
		if (normal.y > 0.f) normal.y *= -1.f;
	}

	//engine pointer, camera (x,y) position
	void draw(olc::PixelGameEngine* e, int cx, int cy)
	{
		olc::Pixel flashColor = { 255, 255, 0, 255 };
		if (t > 0.6f) flashColor = { 100, 100, 255, 255 };
		if (t > 1.f) t = 0.f;

		int px = int(p.x - cx);
		int py = int(p.y - cy);
		e->DrawCircle(px, py, r, flashColor); // main body
		e->DrawCircle(px, py, r - 1, olc::BLUE);
		if (t < 0.5f) flashColor = { 100, 100, 255, 255 };
		else flashColor = olc::YELLOW;
		e->DrawCircle(int32_t(px + o1.x), int32_t(py + o1.y), r1-1, flashColor);
		e->DrawCircle(int32_t(px + o1.x), int32_t(py + o1.y), r1, olc::BLUE);
		e->DrawCircle(int32_t(px + o2.x), int32_t(py + o2.y), r2-1, flashColor);
		e->DrawCircle(int32_t(px + o2.x), int32_t(py + o2.y), r2, olc::BLUE);

		e->DrawLine(int32_t(px), int32_t(py), int32_t(px + (normal.x*r)), int32_t(py + (normal.y*r)));
		lightning.Draw(e);
	}

public:
	vec2  p;             // position
	vec2 o1 = {-20, 20 }; // offset thumb 1
	vec2 o2 = { 20, 20 }; // offset thumb 2

	int r = 10;
	int r1 = 5;
	int r2 = 5;

	int selectedID = -1;
	vec2 normal = { 0.f, 0.f };

private:
	LightningBolt lightning;
	float t = 0.f;
};