#pragma once

#include <olcpixelgameengine.h>

static std::map<int, std::string> StrNum = { {0, "01.png"}, {1, "02.png"}, {2, "03.png"}, {3, "04.png"}, {4, "05.png"}, {5, "06.png"},
											 {6, "07.png"}, {7, "08.png"}, {8, "09.png"}, {9, "10.png"}, {10, "11.png"}, {11, "12.png"} };

enum eAnimState
{
	idle,
	run,
	fall,
	block,
	jump,
	fly
};

class Player
{
public:
	Player() 
	{
		auto path = [](std::string name, int index) { return "Assets/Actors/" + name + StrNum.at(index); };

		// load animations
		for (int i = 0; i < 12; i++) { anim_run[i].LoadFromFile(path("soldier_run", i)); }
		for (int i = 0; i < 9; i++) { anim_look[i].LoadFromFile(path("soldier_look", i)); }
		for (int i = 0; i < 8; i++) { anim_fall[i].LoadFromFile(path("soldier_fall", i)); }
		for (int i = 0; i < 6; i++) { anim_fly[i].LoadFromFile(path("soldier_fly", i)); }
		for (int i = 0; i < 4; i++) { anim_block[i].LoadFromFile(path("soldier_block", i)); }
		// load animation climb;
		// load animation shute
		// load animation < ... >
	}


	void Update(float fElapsedTime)
	{
		if (isJumpTriggered)
		{
			isJumpTriggered = false;
			if ((animState != eAnimState::fall) || (animState != eAnimState::fly) || (animState != eAnimState::jump))
			{
				isJumping = true;
				jumpPos = pos;
			}
		}

		float maxVelx = 120.0; // ~~max:120px p/sec~~
		float maxVely = 2000.f;
		if (isJumping) // ------------------------
		{
			jumpTime += fElapsedTime;

			if (jumpTime > 0.3f)
			{
				jumpTime = 0.f;
				isJumping = false;
				vel.y = 0;
				acc.y = 100;
			}

			// math 
			// https://stackoverflow.com/questions/24900385/c-sdl-gravity-in-platformer
		}
		else
		{   // falling ----------------------------
			if (!isGrounded)
				//acc.y += 0.96f / fElapsedTime; // gravity - wall cling
				//acc.y *= (1.f - fElapsedTime);
				acc.y += 0.981f * fElapsedTime; //fall faster 
			else
			{
				maxVelx = 160;
				maxVely = 260;
			}

		}


		// clamps --------------------------------------------------
		vel += acc * fElapsedTime;
		if (vel.x > maxVelx) vel.x = maxVelx;
		if (vel.x < -maxVelx) vel.x = -maxVelx;
		if (vel.y > maxVely) vel.y = maxVely;
		if (vel.y < -maxVely) vel.y = -maxVely;

		pos += vel * fElapsedTime;       // advance position
		if (pos.y < 0.f) pos.y = 0.f;    // clamp top bounds
		if (pos.x < 0.f)pos.x = 0.f;     // clamp right side bounds
		if (pos.x > 1600.f) pos.x = 1600.f;

		// advance animations ----------------------------------------
		switch (animState)
		{
		case eAnimState::jump:
			indexAnim_fall = 0;
			speed = moveSpeed_run * 1.8f;
			break;
		case eAnimState::idle:
			indexAnim_look += fElapsedTime * animSpeed_look;
			speed = 0.f;
			break;
		case eAnimState::run:
			indexAnim_run += fElapsedTime * animSpeed_run;
			speed = moveSpeed_run;
			break;
		case eAnimState::fall:
			indexAnim_fall += fElapsedTime * animSpeed_fall;
			speed = 0.f;
			break;
		case eAnimState::block:
			if (indexAnim_block < 3.f)
				indexAnim_block += fElapsedTime * animSpeed_block;
			speed = 0.f;
			break;
		case eAnimState::fly:
			indexAnim_fly += fElapsedTime * animSpeed_fly;
			speed = moveSpeed_fly;
			break;
		}
		if (animState != eAnimState::block)
			indexAnim_block = 0.f; // block trigger reset 
	}


	void Draw(olc::PixelGameEngine* e, float cx, float cy)
	{
		// calc new mod per cycle 
		static int nDelay = 0;
		int index;
		vec2 p = pos - vec2(cx, cy);
	
		switch (animState)
		{
		case eAnimState::jump:
			e->DrawSprite(p.x - originOffset_run.x, p.y - originOffset_run.y, &anim_fall[0], 1, facingRight);
			break;
		case eAnimState::idle:
			index = int(indexAnim_look) % 9;
			e->DrawSprite(p.x - originOffset_look.x, p.y - originOffset_look.y, &anim_look[index], 1, facingRight);
			break;
		case eAnimState::run:
			index = int(indexAnim_run) % 12;
			if (index == 0 || index == 3 || index == 6 || index == 9)
			{
				nDelay++;
				if (nDelay == 3)
				{
					nDelay = 0;
					bFootstep = true;
				}
			}
			e->DrawSprite(p.x - originOffset_run.x, p.y - originOffset_run.y, &anim_run[index], 1, facingRight);
			break;
		case eAnimState::fall:
			index = int(indexAnim_fall) % 8;
			e->DrawSprite(p.x - originOffset_look.x, p.y - originOffset_look.y, &anim_fall[index], 1, facingRight);
			break;
		case eAnimState::block:
			index = int(indexAnim_block) % 4;
			//e->DrawCircle(p.x + collisionPos.x, p.y + collisionPos.y, 20);                        // debug collision sphere
			e->DrawCircle(p.x + collisionPos.x, p.y + collisionPos.y, 17, olc::Pixel(50, 78, 255)); // debug collision sphere
			e->DrawCircle(p.x + collisionPos.x, p.y + collisionPos.y, 14, olc::YELLOW);             // debug collision sphere
			e->DrawSprite(p.x - originOffset_block.x, p.y - originOffset_block.y, &anim_block[index], 1, facingRight);
			break;
		case eAnimState::fly:
			index = int(indexAnim_fly) % 6;
			e->DrawSprite(p.x - originOffset_fly.x, p.y - originOffset_fly.y, &anim_fly[index], 1, facingRight);
			break;
		}
		//  e->Draw(int32_t(p.x), int32_t(p.y), olc::YELLOW); // debug position marker
	}


public:
	bool isGrounded = false;

	vec2 pos = { 0.f, 0.f };
	vec2 vel = { 0.f, 0.f };
	vec2 acc = { 0.f, 0.f };

	float speed;
	eAnimState animState = eAnimState::idle;
	bool facingRight = false;

	// sound
	bool bFootstep = false;
	int iSnd[12] = { -17, -17, -16, -16, -14, -14, -22, -22, -15, -15, -24, -24 };
	int iSndCurrent = 0; 

	vec2 collisionPos = { 0.f, -20.f };
	vec2  jumpPos = { 0.f, 0.f };
	float jumpHeight = 50.f;
	float jumpTime = 0.f;
	bool isJumpTriggered = false;
	bool isJumping = false;

private:

	vec2 originOffset_run  = { 20, 47 }; // center, bottom(-1) (anim run)
	vec2 originOffset_look = { 26, 44 };
	vec2 originOffset_fly  = { 20, 55 };
	vec2 originOffset_block = { 35, 44 };

	olc::Sprite anim_run[12];
	float indexAnim_run = 0;
	float animSpeed_run = 15.0f;
	float moveSpeed_run = 15.0f;

	olc::Sprite anim_look[9];
	float indexAnim_look = 0;
	float animSpeed_look = 2.f;
	
	olc::Sprite anim_fall[8];
	float indexAnim_fall = 0;
	float animSpeed_fall = 20;

	olc::Sprite anim_block[4];
	float indexAnim_block = 0;
	float animSpeed_block = 20;

	olc::Sprite anim_fly[6];
	float indexAnim_fly = 0;
	float animSpeed_fly = 10.0f;
	float moveSpeed_fly = 40.0f;

};