#ifndef GAME_STATE_H_
#define GAME_STATE_H_

#include "States/base_state.h"
#include <olcPGEX_Graphics2D.h>

namespace Core {

	enum eStateAction
	{   // tangent actions the system resoponds to
		pause,
		play,
		levelUp,
		win, 
		loose,
		quit,
		none
		// there can be others not listed for hard transition paths
	};

	enum eSoundTypes { 
		explosion, 
		bell, 
		tick, 
		numSoundTypes 
	};

	enum eImageTypes {
		png,
		anim,
		numImageTypes
	};


	class StateManager
	{
	public:
		void initialize(olc::PixelGameEngine* engine, vec2 _clientSize);
		void shutdown();

		void changeState(States::Base_state* _state);
		void pushState(States::Base_state* _state);
		void popState();

		void handleEvents(eStateAction _action);
		void update(float _deltaTime);
		void draw();

		bool isRunning() { return running;  }
		void quit()      { running = false; }

		// sound ----------------------------------------------
		void playSound(int k, int type, double timeOffsetFromNow, double duration);

		// GUI DRIVEN ----------------
		double dVolume    = 0.15; // right and left?                // todo : panning
		double difficulty = 0.5;  // 0.0 --> 1.0 maps to 0.4 --> 2.5 time scale
		double dSlider01  = 0.0;
		double dSlider02  = 0.0;
		// END DRIVEN -----------------

		// core / share -------------------------------------------------------------------------------
		std::vector<std::shared_ptr<States::Base_state>> states;    // library of game state controllers
		std::vector<olc::Sprite*> orbSprites;                       // library of orb pixel art
		std::vector<olc::Sprite*> charSprites;                      // library of player character pixel art

		olc::PixelGameEngine* engine;

	private:
		bool running;
	};
}

#endif
