#pragma once

class Line
{
public:
	vec2 A;
	vec2 B;
	float g;

	Line(vec2 a, vec2 b, float girth = 1.f)
	{
		A = a;
		B = b;
		g = girth;
	}

	void Draw(olc::PixelGameEngine* e, olc::Pixel color, int cx=0, int cy=0)
	{
		e->DrawLine(A.x-cx, A.y-cy, B.x-cy, B.y-cy, color);
	}
};

class LightningBolt
{
public:
	LightningBolt() {}
	std::vector<Line> segments1;
	std::vector<Line> segments2;
	float alpha;
	float duration;
	olc::Pixel tint;

	vec2 vStart;
	vec2 vEnd;

	bool isComplete() { return alpha <= 0; } // auto killer

	LightningBolt(vec2 src, vec2 dst, olc::Pixel color) 
	{
		tint = color;
		alpha = 1.f;
		duration = 0.14f;
		vStart = src;
		vEnd = dst;	
		//CreateBolt(src, dst, 1.8f, &segments2);
	}


	void Update(float fElapsedTime, vec2 start, vec2 end)
	{
		vStart = start;
		vEnd = end;
		static bool toggle = true;
		alpha -= fElapsedTime / duration;
		if (alpha < 0.f)
		{
			if (toggle == true)
			{
				CreateBolt(vStart, vEnd, &segments2);
				toggle = false;
			}
			else {
				CreateBolt(vStart, vEnd, &segments1);
				toggle = true;
			}
			alpha = 0.7f;
		}
	}

	void Draw(olc::PixelGameEngine* e)
	{
		//e->SetPixelMode(olc::Pixel::ALPHA);
		if (alpha <= 0.f) return;
		for (auto seg : segments2)
		{
			olc::Pixel color = olc::Pixel(200.f, 200.f, 180.f);
			//color.a = 255;
			seg.Draw(e, color);
		}
		for (auto seg : segments1)
		{
			tint.r = int(((float)rand() / (float)RAND_MAX) * 255);
			//if (tint.r < 28) tint.r = 28;
			//if (tint.r > 200) tint.r = 250;
			tint.g = int(((float)rand() / (float)RAND_MAX) * 255);
			tint.b = int(((float)rand() / (float)RAND_MAX) * 255);
			if (tint.b < 208) tint.b = 250;
			//tint.a = (alpha + 0.3) * 255;
			seg.Draw(e, tint);
		}
		//e->SetPixelMode(olc::Pixel::NORMAL);
	}

	float prevDisplacement = 0.f;
	void CreateBolt(vec2 src, vec2 dst, std::vector<Line> *segs)
	{
		//segments.push_back(Line(src, dst));
		segs->clear();
		vec2 tangent = dst - src;
		vec2 normal = vec2(tangent.y, -tangent.x).norm();
		float length = tangent.mag();
		int countSegments = int(length / 4);
		float* pNoise = new float[countSegments]();
		for (int i = 0; i < countSegments; i++)
			pNoise[i] = (float)rand() / (float)RAND_MAX;
		
		std::sort(pNoise, pNoise + countSegments, std::less<float>());

		const float sway = 0.1f;
		const float jaggedness = 1.f / sway;
		vec2 prevPoint = src;

		for (int i = 1; i < countSegments; i++)
		{
			float pos = pNoise[i];
			float scale = (length * jaggedness) * (pos - pNoise[i - 1]);
			float envelope = pos > 0.95f ? 20.f * (1.f - pos) : 1.f;
			float displacement = sway * ( (      ((float)rand() / (float)RAND_MAX)*2.f          ) - 1.f );
			displacement -= (displacement - prevDisplacement) * (0.7f - scale);
			displacement += envelope;

			vec2 point = src + (pos * tangent) + (displacement * normal);
			segs->push_back(Line(prevPoint, point));
			prevPoint = point;
			//prevDisplacement = displacement;
		}
		delete[] pNoise;
	}
};




















