// file : base_state.h

#ifndef _engine_core_states_base_state_h_
#define _engine_core_states_base_state_h_

#include <olcpixelgameengine.h>
#include <string>

namespace Core {
	namespace States {

		// abstract
		class Base_state
		{
		public:
			Base_state(void* _manager, std::string _name, vec2 _clientSize)
			{
				pManager = _manager;
				name = _name;
				clientSize = _clientSize;
			}

			virtual void initialize() = 0;
			virtual void shutdown() = 0;

			virtual void pause() = 0;
			virtual void resume() = 0;

			virtual void handleEvents() = 0;
			virtual void update(float _deltaTime) = 0;
			virtual void draw() = 0;

			void* pManager;
			std::string name;
			vec2 clientSize;


		protected:
			Base_state() { /* empty constructor */ }

		};

	}

	struct sAnimInfo
	{
		int id = 0;
		int min_id = 0;      // min texture id
		int max_id = 0;      // max texture id
		float speed = 1.f; // playback speed
		float time = 0.f;  // animation time

		void update(float dt)
		{
			time += dt * speed;
			int dif = max_id - min_id; // start-end inclusive???
			if (time < 0)   time += dif;
			if (time > dif) time -= dif;
			id = min_id + (int(time) % dif);
		}
	};

	// barf:
    // okay, data driven. Yo says, you're welcome in advance
	//float lerp(float p1, float po, float t) { return (1 - t) * po + t * p1; }
}


#endif