// file : game.cpp

#include "game.h"
#include "../GameState.h"
using namespace Core::States;

#include "../Actors/Pipes.h"
Pipes pipes;

#include "../Actors/PowerUp.h"
PowerUp powerups;

// mwk was here.
// https://docs.microsoft.com/en-us/windows/desktop/api/profileapi/nf-profileapi-queryperformancecounter
// http://reedbeta.com/blog/gpu-profiling-101/ 


// debre helper--------------------------
int debreSpawnCounter = 0;
int debreIndexCounter = 0;
int AdvanceDebreIndexConter()
{
	static int currentIndex = -1;
	currentIndex++;
	if (currentIndex == 512)
		currentIndex = 0;

	return currentIndex;
} // -------------------------------------


void Game::initialize()
{
	terrain.Reset();	
	//terrain.CreateMap();                                               // initialize a known map state

	// mwk: set first to dead bug...still looking (19/05/06)
	debreObjs[0].bDead = true;                                           // hack fix...missed somewhere...pffttt...known state

	objPooper.init();                                                    // stop laughing :|

	pipes.init();                                                        // I mean it :D
	pipes.drawCollisionData(&terrain);                                   // change the world with something special and make it known

	powerups.pManager = (Core::StateManager*)pManager;                   // full subsystem per level...ping and let it do it's thing
	powerups.init();

	objBumpers.push_back(std::unique_ptr<Bumper>(new Bumper(80, 200)));  // these are persistent. here you go... make it happen captain
	objBumpers.push_back(std::unique_ptr<Bumper>(new Bumper(280, 200)));
	objBumpers.push_back(std::unique_ptr<Bumper>(new Bumper(480, 200)));

	objPlayerGoal.LoadFromFile("Assets/capture.png");                    // here's our hole. Balls in hole. 
}

void Game::shutdown() {}
void Game::pause()    {}
void Game::resume()   {}

// re run generation procedure
//int oldLevel = -1;
int thisLevel = 0;
void Game::reset(int level)   // ---------------------------------------------------
{                             //  called during level up state transition trigger   |
	isLevelCompleted = false; // ---------------------------------------------------
	thisLevel = 0;

	pipes.pipeLayoutIndex = -1;
	terrain.CreateMap(); // self increments

	// todo : advance keyframe indices

	physicsObjs.clear(); // empty also???

	// small debre ---------------------------------------------------------------
	debreSpawnCounter = 0;
	debreIndexCounter = 0; // -1???
	for (int i = 0; i < 512; i++)
	{
		debreObjs[i].bDead = true;
		debreObjs[i].bExplode = false;
		debreObjs[i].bHitThisFrame = false;
		debreObjs[i].bStable = false;
		debreObjs[i].nBounceCount = 0;
		debreObjs[i].nBounceBeforDeath = -1;
		debreObjs[i].nextPos = vec2(0.f, 0.f);
		debreObjs[i].pos = vec2(0.f, 0.f);
		debreObjs[i].nextVel = vec2(0.f, 0.f);
		debreObjs[i].vel = vec2(0.f, 0.f);
		debreObjs[i].acc = vec2(0.f, 0.f);
	}
	// the player character -------------------------------------------------------
	player.pos = vec2(60.f, 150.f);
	player.vel = vec2(0.f, 0.f);
	player.isGrounded = false;
	
	camera.posx = 0.f;
	camera.posy = 0.f; // DO NOT ADJUST. WILL ACCESS OUTSIDE OF TERRAIN BOUNDS
	// the pooper -----------------------------------------------------------------
	objPooper.currentTimeSlice = -2.f;
	objPooper.objFactory.pos = objPooper.objFactory.nextPos = vec2(820.f, 40.f);
	objPooper.lastKey.px = 820.f;
	objPooper.lastKey.py = 40.f;
	objPooper.nextKey.px = 820.f;
	objPooper.nextKey.py = 40.f;
	objPooper.lastKey.t = -2.f;
	objPooper.nextKey.t = 0.f;
	objPooper.lastKey.aux = 0;
	objPooper.nextKey.aux = 0;
	objPooper.objGoalPin.pos = objPooper.objGoalPin.nextPos = objPooper.objFactory.pos;
	objPooper.objGoalPin.isCarried = true;
	// the pipes ------------------------------------------------------------------
	pipes.pipeLayoutIndex == 5 ? pipes.pipeLayoutIndex = 0 : pipes.pipeLayoutIndex++;
	pipes.drawCollisionData(&terrain);
	// the pipe cap collision data e->DrawSprite(objGpos.x-cx, objGpos.y-cy, &objPlayerGoal);
	for (int j = 0; j < objPlayerGoal.height; j++) {
		for (int i = 0; i < objPlayerGoal.width; i++) {
			if (objPlayerGoal.GetPixel(i, j).a != 0) {
				int pipeIndex = pipes.pipeLayoutIndex * 2;
				int gpX = int(objPlayerGoalPosition[pipeIndex]);
				int gpY = int(objPlayerGoalPosition[pipeIndex + 1]);
				int gpIndexY = (gpY + j) * terrain.nMapWidth;
				terrain.map[gpIndexY + gpX + i] = Terrain::eType::goal; // pipeID = 3;
			}
		}
	}
	// the goal pin ---------------------------------------------------------------
	objPooper.objGoalPin.isCarried = true;
	objPooper.objGoalPin.bStable = false;
	objPooper.objGoalPin.bHitThisFrame = false;
	objPooper.objGoalPin.nBounceCount = 0;
	objPooper.objGoalPin.pos = { 830.f, 50.f };
	objPooper.objGoalPin.vel = { 0.f, 0.f };

	objPooper.objGoalPin.pos = vec2(objPooper.ai.posx, objPooper.ai.posy);
	objPooper.ai.bDropPin = false;
	objPooper.ai.bFoundPipe = false;
	objPooper.ai.fireThisFrame = 0;
	objPooper.ai.targetTime = 1.0f;
	objPooper.ai.m_machine->reset();


	// the user bumpers -----------------------------------------------------------
	vec2 bump = { 250.f, 200.f };
	for (auto &bumper : objBumpers)
	{
		bumper->p = bump;
		bump.x += 250.f;
		bumper->selectedID = -1;
	}

	// todo : power ups

	// todo : ester egg
}

void Game::handleEvents() {}

char* Game::getLevelTimeString()
{
	static char buffer[64];
	// todo : put a cap on game time to avoid overflow?
	int minutes = int(playerGameTime / 60);
	int hours = int(minutes / 60);
	minutes = minutes % 60;
	float seconds = playerGameTime - (hours * 3600) - (minutes * 60);
	if(hours > 0)
		snprintf(buffer, sizeof(buffer), "Time %d:%d:%.1f\n\nLevel %d", hours, minutes, seconds, nLevelNum);
	else if(minutes > 0)
		snprintf(buffer, sizeof(buffer), "Time %d:%.1f\n\nLevel %d", minutes, seconds, nLevelNum);
	else
		snprintf(buffer, sizeof(buffer), "Time %.1f\n\nLevel %d", seconds, nLevelNum);

	return buffer;
} 

struct Point
{
	int x;
	int y;
};

// line segment collisions -------------------------------------------------------
// To find orientation of ordered triplet (p, q, r). 
// The function returns following values 
// 0 --> p, q and r are colinear 
// 1 --> Clockwise 
// 2 --> Counterclockwise 
int orientation(Point p, Point q, Point r)
{
	// See https://www.geeksforgeeks.org/orientation-3-ordered-points/ 
	// for details of below formula. 
	int val = (q.y - p.y) * (r.x - q.x) -
		(q.x - p.x) * (r.y - q.y);

	if (val == 0) return 0;  // colinear 

	return (val > 0) ? 1 : 2; // clock or counterclock wise 
}

// Given three colinear points p, q, r, the function checks if 
// point q lies on line segment 'pr' 
bool onSegment(Point p, Point q, Point r)
{
	if (q.x <= std::max(p.x, r.x) && q.x >= std::min(p.x, r.x) &&
		q.y <= std::max(p.y, r.y) && q.y >= std::min(p.y, r.y))
		return true;

	return false;
}

bool doLineSegmentsIntersect(Point p1, Point q1, Point p2, Point q2)
{
	// Find the four orientations needed for general and 
	// special cases 
	int o1 = orientation(p1, q1, p2);
	int o2 = orientation(p1, q1, q2);
	int o3 = orientation(p2, q2, p1);
	int o4 = orientation(p2, q2, q1);

	// General case 
	if (o1 != o2 && o3 != o4)
		return true;

	// Special Cases 
	// p1, q1 and p2 are colinear and p2 lies on segment p1q1 
	if (o1 == 0 && onSegment(p1, p2, q1)) return true;

	// p1, q1 and q2 are colinear and q2 lies on segment p1q1 
	if (o2 == 0 && onSegment(p1, q2, q1)) return true;

	// p2, q2 and p1 are colinear and p1 lies on segment p2q2 
	if (o3 == 0 && onSegment(p2, p1, q2)) return true;

	// p2, q2 and q1 are colinear and q1 lies on segment p2q2 
	if (o4 == 0 && onSegment(p2, q1, q2)) return true;

	return false; // Doesn't fall in any of the above cases 
}

float hackGameOverTimer = 0.f;

//  _   _  ____  ____     ___  _______ _____
// | | | ||  _ \|  _ \   /   \|__   __| ____|
// | | | || |_| | | \ \ | |_| |  | |  | |_
// | |_| || |__/| |__| ||  _  |  | |  | |___
// |_____||_|   |______||_| |_|  |_|  |_____|
//
void Game::update(float fElapsedTime)
{
	if (fElapsedTime > 0.15f) fElapsedTime = 0.15f; // time spike clamp
	playerGameTime += fElapsedTime;

	// circle collisions ----------------------- 
	auto DoCirclesOverlap = [](float x1, float y1, float r1, float x2, float y2, float r2)
	{
		return fabs((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2)) <= (r1 + r2) * (r1 + r2);
	};

	auto IsPointInCircle = [](float x1, float y1, float r1, float px, float py)
	{
		return fabs((x1 - px)*(x1 - px) + (y1 - py)*(y1 - py)) < (r1 * r1);
	};

	Core::StateManager* manager = (StateManager*)pManager;
	olc::PixelGameEngine* e = manager->engine;

	if (isLevelCompleted)
	{   // level has triggered end. advance state.
		isLevelCompleted = false;
		nLevelNum++;
		playerGameTime = 0;
		manager->handleEvents(Core::eStateAction::levelUp); // reset control path
	}

#pragma region user_keyboard_input__movement

	// set player animation state on key press --------------------------------
	bool isMoving = false;

	// directional (pressed) ***********************************************
	if(e->GetKey(olc::RIGHT).bPressed || e->GetKey(olc::D).bPressed)  //  //
	{                                                                 //  //
		player.vel.x += 10.f;                                         //  //
		player.acc.x = 1500.f;                                        //  //
		player.facingRight = true;                                    //  //
	}                                                                 //  //
	if (e->GetKey(olc::LEFT).bPressed || e->GetKey(olc::A).bPressed)  //  //
	{                                                                 //  //
		player.vel.x -= 10.f;                                         //  //
		player.acc.x = -1500.f;                                       //  // hard coded explosive value?
		player.facingRight = false;                                   //  //
	}   // directional (held) **********************************************
	if (e->GetKey(olc::LEFT).bHeld || e->GetKey(olc::A).bHeld)        //  //
	{	                                                              //  //
		isMoving = true;                                              //  //
		player.acc.x -= 50.0f;                                        //  //
		player.isGrounded ? player.animState = eAnimState::run : player.animState = eAnimState::fall;
	}                                                                 //  //
	if (e->GetKey(olc::RIGHT).bHeld || e->GetKey(olc::D).bHeld)       //  //
	{	                                                              //  //
		//int here = int(player.pos.y - 10 * terrain.nMapWidth + player.pos.x);
		isMoving = true;                                              //  //
		player.acc.x += 50.0f;// fElapsedTime                         //  //
		player.isGrounded ? player.animState = eAnimState::run : player.animState = eAnimState::fall;
	} // *******************************************************************
	if (isMoving == false)                                            //  //
	{                                                                 //  //
		player.isGrounded ? player.animState = eAnimState::idle : player.animState = eAnimState::fall;
	} // directional (release) *********************************************
	if (e->GetKey(olc::RIGHT).bReleased || e->GetKey(olc::LEFT).bReleased ||
		e->GetKey(olc::A).bReleased || e->GetKey(olc::D).bReleased)   //  //
	{                                                                 //  //
		player.vel.x = 0.f;                                           //  //
		player.acc.x = 0.f;                                           //  //
	} // end directional (press, hold, release) ****************************


	// Jumping -------------------------------------------------------------------
	bool isJumpPressed = (e->GetKey(olc::UP).bPressed || e->GetKey(olc::W).bPressed);
	bool isJumpHeld = (e->GetKey(olc::UP).bHeld || e->GetKey(olc::W).bHeld);
	bool isJumpReleased = (e->GetKey(olc::UP).bReleased || e->GetKey(olc::W).bReleased);

	if (isJumpPressed && player.isGrounded)
	{	                                                       // is against something to push off (grounded == wall)
			player.isGrounded = false;
			player.isJumpTriggered = true;                     // signal to begin action
			player.vel.y = -250.f;
			player.acc.y = -100.f;
		
		// https://www.youtube.com/watch?v=hG9SzQxaCm8 GDC16 better jumping math
		// http://archive.gamedev.net/archive/reference/business/features/shareprof/  shareware amatures vrs professionals
		// https://www.eurogamer.net/articles/2018-07-20-captain-toad-pixel-toad-locations-episode-1-2-3-5497 sample rotation camera
		
		// https://gamedevelopment.tutsplus.com/tutorials/basic-2d-platformer-physics-part-1--cms-25799
		// todo : more or good enough 
	}
	
	// if (isJumpHeld) { ; }

	if (isJumpReleased )
	{
		player.acc.y = 250.f;
	}

	// blocking ---------------------------------------------------------------	
	if (e->GetKey(olc::DOWN).bPressed || e->GetKey(olc::S).bPressed)
		player.animState = eAnimState::block;
	if (e->GetKey(olc::DOWN).bHeld || e->GetKey(olc::S).bHeld)
		player.animState = eAnimState::block;
	if (e->GetKey(olc::DOWN).bReleased || e->GetKey(olc::S).bReleased)
		player.animState = eAnimState::idle;

#pragma endregion

	if (e->GetKey(olc::ESCAPE).bPressed) { manager->popState(); } // escape to main menu
	if (e->GetKey(olc::M).bReleased) { terrain.CreateMap(); } // change terrain map	

	if (e->GetKey(olc::N).bPressed) { isLevelCompleted = true; }// manager->handleEvents(Core::eStateAction::levelUp); terrain.CreateMap();
	if (e->GetKey(olc::B).bPressed) { objPooper.objGoalPin.isCarried = false; }
	if (e->GetKey(olc::F1).bPressed) { isDebug_player = !isDebug_player; }

#pragma region user_input_mouse_NOT_USED_
	// mouse interruption (debug injection) ------------------------------------
	/*int mX = e->GetMouseX();
	int mY = e->GetMouseY();
	if (e->GetMouse(0).bReleased)
	{
		vec2 viewWorldPos = { mX + camera.posx, mY + camera.posy };
		Dummy* p = new Dummy(viewWorldPos.x, viewWorldPos.y);
		p->id = physicsObjs.size();
		physicsObjs.push_back(std::unique_ptr<Dummy>(p));
	}*/
#pragma endregion

#pragma region camera
	// camera -------------------------------------------------
	// clamp to terrain width
	int tw = terrain.nMapWidth;  // terrain width
	int th = terrain.nMapHeight; // terrain height
	int sw = e->ScreenWidth();   // screen width
	int sh = e->ScreenHeight();  // screen height
	int px = (int)player.pos.x;       // player x position
	int py = (int)player.pos.y;       // player y position
	float offset = camera.posx + (sw / 2.0f);
	int maxX = tw - sw;

	float proximityScale = abs(px - offset) / 100.f; 
	if (proximityScale > 1.0f) proximityScale = 1.f;
	if (proximityScale < 0.1f) proximityScale = 0.f; // smooth out camera movement
	if (px > offset + 2) camera.posx += camera.scrollSpeed * fElapsedTime * proximityScale;
	if (px < offset - 2) camera.posx -= camera.scrollSpeed * fElapsedTime * proximityScale;
	// do nothing if equivalent
	if (camera.posx < 0.0f) camera.posx = 0.0; // clamp to positive positions. 
	if (camera.posx > maxX) // clamp to right bounds 
	{
		//if(checkpointOkay) allow posx into boss fight
		camera.posx = float(tw - sw);
		//player.pos.x -= terrain.nMapWidth;
		//player.transform.Translate( -terrain.nMapWidth, 0.f); 
	}
#pragma endregion 

#pragma region object_gravity_and_terrain
	// gravity and terrain ---------------------------------------
	for (auto &obj : physicsObjs)
	{
		if (obj->bDead) {
			obj->vel.y = 0.f;
			continue;
		}

		// apply force ....................................
		float gravity = 9.81f * obj->mass * fElapsedTime;
		vec2 force = { 0.f, 0.f }; // todo : mass / size maybe / etc..  obj->acc
		obj->nextVel.x += force.x;
		obj->nextVel.y += (gravity)+force.y;
		// clamp velocity .................................
		float terminalVelocity = 2.f;
		if (obj->nextVel.x > terminalVelocity) obj->nextVel.x = terminalVelocity;
		if (obj->nextVel.x < -terminalVelocity) obj->nextVel.x = -terminalVelocity;
		if (obj->nextVel.y > terminalVelocity) obj->nextVel.y = terminalVelocity;
		if (obj->nextVel.y < -terminalVelocity) obj->nextVel.y = -terminalVelocity;
		// calculate desired position
		obj->nextPos = obj->pos + obj->nextVel * fElapsedTime * 50.f;
		// potential positions established and next desired state set (free form)			
		obj->bHitThisFrame = false; // reset for next run (hmmm...not sure i want or need)

		// wrap horizontal
		//if (obj->nextPos.x < 0.f) obj->nextPos.x += terrain.nMapWidth;
		//if (obj->nextPos.x > terrain.nMapWidth) obj->nextPos.x -= terrain.nMapWidth;

		// check against terrain (sudo raycast)
		vec2 reactionAccumulator = { 0.f, 0.f }; // sum all positive hit vectors
		int numHits = 0;
		if ((obj->nextPos.y + obj->radius) < terrain.nMapHeight)
		{
			// assuming a positive downward velocity (only test lower hemisphere)
			int xL = int(obj->nextPos.x - obj->radius);   // left index
			int xC = int(xL + obj->radius);               // ---- center vertical ----
			int xR = int(xC + obj->radius);               // right index
			int yT = int(obj->nextPos.y);                 // hemi slice center top
			int yB = int(yT + obj->radius);               // bottom index
			float objR2 = obj->radius * obj->radius; // radius squared
			for (int i = yB; i > yT; i--)
			{	// scanline search for terrain (bottom up)
				int scanlineIndex = i * terrain.nMapWidth;
				for (int j = xC; j > xL; j--)
				{   // scan center to left- break on first hit
					if (terrain.map[scanlineIndex + j] == Terrain::eType::land ||
						terrain.map[scanlineIndex + j] == Terrain::eType::pipe ||
						terrain.map[scanlineIndex + j] == Terrain::eType::goal)
					{
						vec2 v = obj->pos - vec2(float(j), float(i));
						if ((v.x * v.x) + (v.y * v.y) < objR2)
						{
							reactionAccumulator += v;
							numHits++;
							break;
						}
					}
				}
				for (int j = xC; j < xR; j++)
				{   // scan right to center - break on first hit
					if (terrain.map[scanlineIndex + j] == Terrain::eType::land ||
						terrain.map[scanlineIndex + j] == Terrain::eType::pipe ||
						terrain.map[scanlineIndex + j] == Terrain::eType::goal)
					{
						vec2 v = obj->pos - vec2(float(j), float(i));
						if ((v.x * v.x) + (v.y * v.y) < objR2)
						{
							reactionAccumulator += v;
							numHits++;
							break;
						}
					}
				}
			}
			// resolve accumulated reaction
			if (numHits > 0)
			{
				int here = (obj->nextPos.y*terrain.nMapWidth) + obj->nextPos.x;
				if (terrain.map[here] == Terrain::water)
				{
					obj->nBounceCount = 4; // hack activator (todo : maybe should be 2...rough cleaning pass 05-23-2019mwk)
					obj->bHitThisFrame = true;
					obj->nextVel *= 0.9f;
				}
				else {
					vec2 unitReaction = reactionAccumulator.norm();
					obj->vel = unitReaction * obj->nextVel.mag();
					obj->nextVel = obj->vel * 0.94f; // decay

					obj->bHitThisFrame = true;
					float rnd_i = (float)rand() / (float)RAND_MAX;
					manager->playSound(-int(rnd_i * 11), eSoundTypes::bell, 0.0, 0.009);
				}
				numHits = 0;
			}


			// object position (wrap) terrain screen 
			//if (obj->nextPos.x > terrain.nMapWidth) obj->nextPos.x -= terrain.nMapWidth;
			//if (obj->nextPos.x < 0) obj->nextPos.x += terrain.nMapWidth;
		}
		else {
			obj->bDead = true;
		}

		// end terrain check -----------------------

		// check for sweep collisions objects against objects (from this step per object)
		// object object collision
		for (auto &target : physicsObjs)
		{
			if (target->bDead) continue;
			if (obj->id != target->id) // compare only once per pair
			{
				if (DoCirclesOverlap(obj->nextPos.x, obj->nextPos.y, obj->radius, target->nextPos.x, target->nextPos.y, target->radius))
				{
					vec2 vec = obj->nextPos - target->nextPos;
					float dist = sqrtf(vec.x*vec.x + vec.y*vec.y);
					float halfOverlap = (dist - obj->radius - target->radius) * 0.5f;
					// static displacement (resolve to touched)
					obj->nextPos -= halfOverlap * vec / dist;
					target->nextPos += halfOverlap * vec / dist;

					//target->color = olc::WHITE;
					obj->bHitThisFrame = true;
				}
			}
		}
	}
#pragma endregion

#pragma region the_water
	// temp spawner
	static int watercount = 0;
	int p = int((objHoseBit.pos.y + objHoseBit.outOffset.y) * 1600 + (objHoseBit.pos.x + objHoseBit.outOffset.x));
	watercount++;
	if (watercount > 20)
	{
		watercount = 0;
		for (int i = 0; i < 4; i++)
		{
			terrain.map[p - 2] = 2;
			terrain.map[p + 0] = 2;
			terrain.map[p + 2] = 2;
			p += terrain.nMapWidth * 2;
		}
	}
#pragma endregion

#pragma region the_pooper
	// the pooper
	// update perceptions
	//objPooper.isPlayerMoving = player.animState == eAnimState::run ? true : false;
	//objPooper.tension = player.pos.x - objPooper.objFactory.pos.x;

	objPooper.ai.playerPos = player.pos.x;
	objPooper.ai.bumper01Pos = objBumpers[0]->p.x;
	objPooper.ai.bumper02Pos = objBumpers[1]->p.x;
	objPooper.ai.bumper03Pos = objBumpers[2]->p.x;
	objPooper.ai.posx = objPooper.objFactory.pos.x;
	objPooper.ai.posy = objPooper.objFactory.pos.y;



	int trigger = objPooper.update(fElapsedTime, physicsObjs);
	if (trigger)
	{  
		Dummy* p; // factory the shit out of it.
		switch (trigger)
		{
		case 1: // fire left
			p = new Dummy(objPooper.objFactory.pos.x, objPooper.objFactory.pos.y);
			p->nextPos = p->pos;
			p->nextVel = p->vel = { -1.f, 0.f };
			p->id = physicsObjs.size();
			physicsObjs.push_back(std::unique_ptr<Dummy>(p));
			break;
		case 2: // fire right
			p = new Dummy(objPooper.objFactory.pos.x, objPooper.objFactory.pos.y);
			p->nextPos = p->pos;
			p->nextVel = p->vel = { 1.f, 0.f };
			p->id = physicsObjs.size();
			physicsObjs.push_back(std::unique_ptr<Dummy>(p));
			break;

			// todo : more cases? fire right? change action? 

		}

		manager->playSound(36, Core::eSoundTypes::explosion, 0.0, 1.5);
	}
#pragma endregion

#pragma region the_goal_pin
	/*int pos_below = int((objPooper.objGoalPin.pos.y*terrain.nMapWidth) + objPooper.objGoalPin.pos.x + terrain.nMapWidth);
	while (terrain.map[pos_below] == Terrain::eType::air)   // 1)
	{   // can we launch for the win
		if (terrain.map[pos_below] == Terrain::eType::pipe)  // drop the pin?
		{
			objPooper.objGoalPin.isCarried = false;
			objPooper.objGoalPin.vel.x = objPooper.objGoalPin.nextVel.x = -1.f;
			break;
		}
		pos_below += terrain.nMapWidth;
	}*/

	objPooper.objGoalPin.isCarried = !objPooper.ai.bDropPin; // ai perception
	if (objPooper.objGoalPin.isCarried == false)
	{	                                                     // falling
		objPooper.objGoalPin.acc.y += 0.981f * objPooper.objGoalPin.mass * fElapsedTime;
		// calculate this move
		vec2 nextPos = objPooper.objGoalPin.pos + ((objPooper.objGoalPin.vel + objPooper.objGoalPin.acc) * fElapsedTime);
		int iGx = int(nextPos.x);
		int iGy = int(nextPos.y) - 3; // bury it
		if (iGy < 0) iGy = 0;

		int i = (iGy * terrain.nMapWidth) + iGx;
		switch (terrain.map[i])
		{
		case 0: // open air, apply move
			objPooper.objGoalPin.pos = nextPos;
			objPooper.objGoalPin.vel += objPooper.objGoalPin.acc * fElapsedTime;
			if (terrain.map[i + (4 * terrain.nMapWidth)] == 0)
				objPooper.objGoalPin.bStable = false;
			break;
		case 1: // land mass, halt and make noise
			objPooper.objGoalPin.vel *= (1.f - fElapsedTime); // decay
			objPooper.objGoalPin.vel.y = 0.f;
			objPooper.objGoalPin.acc.y = 0.f;

			if (!objPooper.objGoalPin.bStable)
				manager->playSound(0, Core::eSoundTypes::tick, 0.0, 0.005);
			objPooper.objGoalPin.bStable = true;
			break;
		case 2: // water
			objPooper.objGoalPin.pos = nextPos;
			objPooper.objGoalPin.vel.y += 1;// objPooper.objGoalPin.acc * fElapsedTime*0.5f;
			if (terrain.map[i + (4 * terrain.nMapWidth)] == 0)
				objPooper.objGoalPin.bStable = false;
		}

		hackGameOverTimer += fElapsedTime;
		if (hackGameOverTimer > 10.f)
		{
			manager->playSound(1, Core::eSoundTypes::tick, 0.0, 0.005);
			isLevelCompleted = true;
		}

	}
	else
	{	// carried by the ball pooper
		objPooper.objGoalPin.pos = objPooper.objFactory.pos + vec2(0.f, 19);
		objPooper.objGoalPin.vel = { 0.f, 0.f };
	}
#pragma endregion

#pragma region particle_system
	auto RandomExplosionColor = []()
	{
		switch (rand() % 5)
		{
		case 0: // yellow
			return olc::Pixel(255, 240, 5, 255);
		case 1: // red
			return olc::Pixel(217, 0, 0, 255);
		case 2: // orange
			return olc::Pixel(255, 145, 2, 255);
		case 3: // dark orange
			return olc::Pixel(255, 77, 0, 255);
		default:
			return olc::WHITE;
		}
	};

	// last chance run through game objects
	for (auto &obj : physicsObjs)
	{
		obj->pos = obj->nextPos; // todo : reset somewhere 
		if (obj->bHitThisFrame)
		{
			obj->bHitThisFrame = false;
			obj->nBounceCount++; // remember, bounce to die
			if (obj->nBounceCount == obj->nBounceBeforDeath)
			{
				float rndX, rndY;
				float scale = 0.25f;
				for (int i = 0; i < 20; i++)
				{
					debreIndexCounter = AdvanceDebreIndexConter();
					rndX = ((((float)rand() / (RAND_MAX))*2.f) - 1.f) * scale; // (-1 to +1)*scale
					rndY = -((float)rand() / (RAND_MAX)) * scale * 2.2f; // 0 to -scale w/intensity modifier
					debreObjs[debreIndexCounter].pos = obj->pos;
					debreObjs[debreIndexCounter].vel = { rndX, rndY };
					debreObjs[debreIndexCounter].nextVel = { rndX, rndY };
					debreObjs[debreIndexCounter].bDead = false;
					debreObjs[debreIndexCounter].bExplode = false;
					debreObjs[debreIndexCounter].bHitThisFrame = false;
					debreObjs[debreIndexCounter].id = i;
					debreObjs[debreIndexCounter].mass = 0.12f;
					debreObjs[debreIndexCounter].radius = 0.05f;
					debreObjs[debreIndexCounter].nBounceCount = 0;
					debreObjs[debreIndexCounter].nBounceBeforDeath = 1;
					debreObjs[debreIndexCounter].type = 1;
					debreObjs[debreIndexCounter].color = RandomExplosionColor();
				}
				for (int i = 0; i < 20; i++)
				{   // type 2 single yellow pixel particle
					debreIndexCounter = AdvanceDebreIndexConter();
					rndX = ((((float)rand() / (RAND_MAX))*2.f) - 1.f) * scale; // (-1 to +1)*scale
					rndY = -((float)rand() / (RAND_MAX)) * scale * 2.2f; // 0 to -scale w/intensity modifier
					debreObjs[debreIndexCounter].pos = obj->pos;
					debreObjs[debreIndexCounter].vel = { rndX, rndY };
					debreObjs[debreIndexCounter].nextVel = { rndX, rndY };
					debreObjs[debreIndexCounter].bDead = false;
					debreObjs[debreIndexCounter].bExplode = false;
					debreObjs[debreIndexCounter].bHitThisFrame = false;
					debreObjs[debreIndexCounter].id = i;
					debreObjs[debreIndexCounter].mass = 0.06f;
					debreObjs[debreIndexCounter].radius = 0.003f;
					debreObjs[debreIndexCounter].nBounceCount = 0;
					debreObjs[debreIndexCounter].nBounceBeforDeath = 1;
					debreObjs[debreIndexCounter].type = 0;
					debreObjs[debreIndexCounter].color = olc::YELLOW;
				}
				obj->bDead = true;
				terrain.Carve(obj->nextPos, obj->radius, 1.5f);
				manager->playSound(0, eSoundTypes::explosion, 0.0, 0.7);
			}
		}
	}


	// particle life span controller --------------------------------------
	for (int i = 0; i < 512; i++)
	{
		if (debreObjs[i].bDead) continue;

		/*float gravity =*/debreObjs[i].acc.y += 9.81f * debreObjs[i].mass * fElapsedTime;
		debreObjs[i].nextVel += debreObjs[i].acc * fElapsedTime;// gravity;
		debreObjs[i].nextPos = debreObjs[i].pos + debreObjs[i].nextVel; // added time

		debreObjs[i].pos = debreObjs[i].nextPos;
		if (debreObjs[i].pos.y > terrain.nMapHeight)
		{
			debreObjs[i].bDead = true; // kill z
			debreObjs[i].acc = vec2(0.f, 0.f);
		}

		if (terrain.map[int(debreObjs[i].pos.y) * terrain.nMapWidth + int(debreObjs[i].pos.x)] == Terrain::eType::land ||
			terrain.map[int(debreObjs[i].pos.y) * terrain.nMapWidth + int(debreObjs[i].pos.x)] == Terrain::eType::pipe)
		{
			debreObjs[i].nBounceCount++;
			debreObjs[i].nextVel.y *= -1.f;
			debreObjs[i].acc = vec2(0.f, 0.f);
		}

		if (debreObjs[i].nBounceCount == debreObjs[i].nBounceBeforDeath)
		{
			terrain.Carve(debreObjs[i].pos, 1.f, 1.f);
			debreObjs[i].bDead = true;
			debreObjs[i].acc = vec2(0.f, 0.f);
		}
	}
#pragma endregion

#pragma region the_player
	// player update -----------------------------------------------------------

	player.Update(fElapsedTime);

	unsigned int terrainIndex_here = player.pos.x;
	for (int elev = 0; elev < terrain.nMapHeight; elev++)
	{	// shoot ray straight down and keep character legal on the terrain
		if (terrain.map[terrainIndex_here] == Terrain::eType::land ||
			terrain.map[terrainIndex_here] == Terrain::eType::pipe ||
			terrain.map[terrainIndex_here] == Terrain::eType::goal)
		{   // land behavior. matter can not share the same space.
			if (player.isGrounded)
			{
				if(player.pos.y > elev)
					player.pos.y = elev;
				else
				{
					player.acc.y += 0.98f / fElapsedTime; // gravity 
				}
			}
			else
			{   // falling
				if (player.pos.y > elev)
				{
					player.isGrounded = true;
					player.pos.y = elev;
					player.vel.y = 0.f;
					player.acc.y = 0.f;
				}
				else
					player.acc.y += 0.98f / fElapsedTime;
			}
			break;
		}
		else if (terrain.map[terrainIndex_here] == Terrain::eType::water)
		{
			// this is okay...what ever it is cut by 1/3 while we're in it
			player.vel *= 0.25f; // slow down in water (might be a burst when exiting +++++acc)
		}

		terrainIndex_here += terrain.nMapWidth;
	}

	
	

	// little pile in the corner still ;p
	
	//player.vel *= 0.98f;// (1.f - fElapsedTime); // decay
	//player.transform.Translate(player.vel.x, player.vel.y);
	// check the player character against the balls --------------------------------------
	//player.transform.Forward(0.f, 0.f, player.collisionPos.x, player.collisionPos.y);
	//player.collisionPos += vec2(28, 23); // collision sphere offset
	for (auto &obj : physicsObjs)
	{
		if (player.animState == eAnimState::block)
		{
			//float dist = (obj->nextPos-player.collisionPos).mag() - obj->radius;
			float dist = (obj->nextPos - player.pos - player.collisionPos).mag() - obj->radius;
			if (dist < 20.f)           // should be checking the squared distance
			{
				// apply the smack force
				obj->nextVel = (obj->pos - player.pos).norm() * 1.6f;
			}
		}
		if (obj->pos.y < 0.f)
		{   // lower bounds y-axis limiter / reflector off ceiling
			obj->pos.y = obj->nextPos.y = 1.f;
			obj->nextVel.y *= -1.f;
		}
	}
#pragma endregion

#pragma region the_powerups
	// power up ---------------------------------------------------------------------------
	powerups.update(fElapsedTime);
	for (auto &pup : powerups.powerUps)
	{
		if (pup.active)
		{
			if (abs(player.pos.y - pup.posY) < 30)
			{
				if (abs(player.pos.x - pup.posX) < 30)
				{
					manager->dSlider02 *= 1.2f;
					for (int i = 0; i < 60; i++)
					{   // type 2 single yellow pixel particle
						debreIndexCounter = AdvanceDebreIndexConter();
						float rndX = ((((float)rand() / (RAND_MAX))*2.f) - 1.f) * 1.f;// scale; // (-1 to +1)*scale
						float rndY = -((float)rand() / (RAND_MAX)) * 0.75f/*scale*/ * 2.2f; // 0 to -scale w/intensity modifier
						debreObjs[debreIndexCounter].pos = vec2(pup.posX, pup.posY);
						debreObjs[debreIndexCounter].vel = { rndX, rndY };
						debreObjs[debreIndexCounter].nextVel = { rndX, rndY };
						debreObjs[debreIndexCounter].bDead = false;
						debreObjs[debreIndexCounter].bExplode = false;
						debreObjs[debreIndexCounter].bHitThisFrame = false;
						debreObjs[debreIndexCounter].id = i;
						debreObjs[debreIndexCounter].mass = 0.06f;
						debreObjs[debreIndexCounter].radius = 0.003f;
						debreObjs[debreIndexCounter].nBounceCount = 0;
						debreObjs[debreIndexCounter].nBounceBeforDeath = 1;
						debreObjs[debreIndexCounter].type = 0;
						debreObjs[debreIndexCounter].color = olc::YELLOW;
					}
					pup.active = 0;
				}
			}
		}
	}
#pragma endregion

#pragma region the_bumpers
	// the bumper (do picking and lightning update) ---------------------------------------
	for (auto &bumper : objBumpers)
	{
		bumper->update(fElapsedTime, vec2(camera.posx, camera.posy));

		// auto reset to ground plane
		if (bumper->selectedID == -1)
		{   // not picking or dragging (do the free fall, snap back or bubble up)
			int bmI_here = ((bumper->p.y*terrain.nMapWidth) + bumper->p.x) + (bumper->r*terrain.nMapWidth);
			int bmI_above = bmI_here - terrain.nMapWidth;
			int bmI_below = bmI_here + terrain.nMapWidth;                               // could do :  a bobbing effect for water ball 
			if (terrain.map[bmI_here] > 0 && terrain.map[bmI_above] > 0) bumper->p.y--; // under ground bubble up and ***floating***
			else
			{
				if (terrain.map[bmI_here] == Terrain::eType::air && terrain.map[bmI_below] == Terrain::eType::air) bumper->p.y++;
				bmI_here = ((bumper->p.y + bumper->o1.y)*terrain.nMapWidth) + (bumper->p.x + bumper->o1.x) + (bumper->r1*terrain.nMapWidth);
				if (terrain.map[bmI_here] > 0) bumper->o1.y--; // anything but air 
				bmI_here = ((bumper->p.y + bumper->o2.y)*terrain.nMapWidth) + (bumper->p.x + bumper->o2.x) + (bumper->r2*terrain.nMapWidth);
				if (terrain.map[bmI_here] > 0) bumper->o2.y--; // anything but air
			}
			if (bumper->o1.x > 40) bumper->o1.x--;
			if (bumper->o1.x < -40) bumper->o1.x++;
			if (bumper->o1.y > 40) bumper->o1.y--;
			if (bumper->o1.y < -40) bumper->o1.y++; // might be better to clamp to a enlarged radius instead of this box
			if (bumper->o2.x > 40) bumper->o2.x--;
			if (bumper->o2.x < -40) bumper->o2.x++;
			if (bumper->o2.y > 40) bumper->o2.y--;
			if (bumper->o2.y < -40) bumper->o2.y++;

			// could do : animated radius wave max indicator thing
		}
		// do the actual line segment test
		// on collide, no reposition is required. test is from center circle to forward mag.
		// on collide, forward is reversed so there is an entire radius space now....all good...
		for (auto &obj : physicsObjs)
		{
			if (obj->bDead) continue;
			// test against active physics objects
			// line segment |ab| <---> line segment |cd| intersection
			Point ap1 = { obj->pos.x, obj->pos.y };
			Point ap2 = { ap1.x + obj->nextVel.x*obj->radius, ap1.y + obj->nextVel.y*obj->radius }; // forward movement segment
			Point bp1 = { bumper->p.x + bumper->o1.x, bumper->p.y + bumper->o1.y }; // root pos + offset1
			Point bp2 = { bumper->p.x + bumper->o2.x, bumper->p.y + bumper->o2.y }; // root pos + offset2

			if (doLineSegmentsIntersect(ap1, ap2, bp1, bp2))
			{	// vR = v - 2 * (v dot n) * n // normalized n
				vec2 vecObj = obj->nextVel;
				vec2 vecReflect = vecObj - 2.f * (vecObj.dot(bumper->normal)) * bumper->normal;

				obj->nextVel = vecReflect * 1.1f;
				obj->nextPos = obj->pos;

				manager->playSound(0, eSoundTypes::explosion, 0.0, 0.7);
			}
		}
	}
#pragma endregion

	// could do : puzzle // climbing...might not happen...

#pragma region the_end_goal

	// end goal ---------------------------------------------------------------------------
	vec2 gp = vec2(objPlayerGoalPosition[(pipes.pipeLayoutIndex*2)], objPlayerGoalPosition[(pipes.pipeLayoutIndex*2+1)]) + vec2(21, 3);
	for (auto &obj : physicsObjs)
	{
		// test for the win
		if ((obj->pos - gp).mag() < obj->radius * 2.f)
		{
			isLevelCompleted = true;
		}
	}
	goalAnimTick += fElapsedTime;
	if (goalAnimTick > 1.0f) goalAnimTick = 0.f;

#pragma endregion

}


//  ____    ____    ___ __       ___
// |  _ \  |  _ \  /   \  \  _  /  /
// | | \ \ | |_| || |_| |  \/ \/  /
// | |__| || |__ \|  _  |\   _   /
// |______||_|  \_|_| |_| \_/ \_/
//
void Game::draw()
{
	Core::StateManager* manager = (StateManager*)pManager;
	olc::PixelGameEngine* e = manager->engine;

	// do last minute iteration over the landscape row chunks to clean strays
	// this routine will leave one pixel peaks and left and right land slides
#pragma region last_chance_terrain_stabalizer_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_

	if (terrain.isStable == true) // not used? always fires at 100 pixel row chunks
	{
		static int tty = terrain.nMapHeight - 1; // search row cursor location (y axis marker)
		for (int n = 0; n < 200; n++) // update count rows of pixels
		{
			tty--; // advance on start for the iteration of the row
			for (int x = 0; x < terrain.nMapWidth; x++)
			{   // scan the single row this frame
				int index = (tty * terrain.nMapWidth) + x; // this location data index
				int indexDown = index + terrain.nMapWidth; // the pixel below
				int indexUp = index - terrain.nMapWidth;
				if (terrain.map[index] != Terrain::eType::air)                          // are we anything but air
				{   // we're a pixel. is someone not below me?
					if (terrain.map[indexDown] == Terrain::eType::air)                  // is air below?
					{	// fall - swap down one pixel
						terrain.map[indexDown] = terrain.map[index];
						terrain.map[index] = Terrain::eType::air;
					} // no, is someone down and right/left?
					else if (terrain.map[index] == Terrain::eType::water)               // are we water?
					{
						if (rand())
						{
							if (terrain.map[indexDown + 1] == Terrain::eType::air)      // is air to the right and down
							{
								terrain.map[indexDown + 1] = terrain.map[index];
								terrain.map[index] = Terrain::eType::air;
							}
						}
						else if (terrain.map[indexDown - 1] == Terrain::eType::air)     // is air to the left and down
						{
							terrain.map[indexDown - 1] = terrain.map[index];
							terrain.map[index] = Terrain::eType::air;
						}
					} // ---------------------------------------------------------------- end check for air and water down

					if (terrain.map[index] == Terrain::eType::land)                     // are we land ?
					{   // dirt slide
						float rnd = (float)std::rand() / (float)RAND_MAX;
						bool bChaos = rnd > 0.95 ? true : false;   // determine chaos for horizon movement
						if (terrain.map[indexDown - 1] == Terrain::eType::air && bChaos == true)
						{ // settle left - swap (down/left) one pixel
							terrain.map[index] = Terrain::eType::air;
							terrain.map[indexDown - 1] = Terrain::eType::land;
						} // no, is someone down and right?
						else if (terrain.map[indexDown + 1] == Terrain::eType::air && bChaos == true)
						{ // settle right - swap (down/right) one pixel
							terrain.map[index] = Terrain::eType::air;
							terrain.map[indexDown + 1] = Terrain::eType::land;
						}
						if (terrain.map[indexDown - 1] == Terrain::eType::water && bChaos == true)
						{ // settle left - swap (down/left) one pixel
							terrain.map[index] = Terrain::eType::water;
							terrain.map[indexDown - 1] = Terrain::eType::land;
						} // no, is someone down and right?
						else if (terrain.map[indexDown + 1] == Terrain::eType::water && bChaos == true)
						{ // settle right - swap (down/right) one pixel
							terrain.map[index] = Terrain::eType::water;
							terrain.map[indexDown + 1] = Terrain::eType::land;
						}
						else if (terrain.map[indexDown] == Terrain::eType::water)
						{
							terrain.map[index] = Terrain::eType::water;
							terrain.map[indexDown] = Terrain::eType::land;
						}
					} // --------------------------------------------------------------- end land random chaos movement

					if (terrain.map[index] == Terrain::eType::water)                  // are we water ?
					{	// water slide
						int sign;                                           // needs something just a little more
						tty % 2 == 0 ? sign = -1 : sign = 1;
						if (terrain.map[index + sign] == Terrain::eType::air) // not quite...tends to lean left when it drops
						{ // flow left - swap (left) one pixel
							terrain.map[index + sign] = Terrain::eType::water;
							terrain.map[index] = Terrain::eType::air;
						}
					} // --------------------------------------------------------------- end water flow movement

					if (terrain.map[index] == Terrain::eType::pipe)
					{	// water spray ---------------------------------
						// check for pipe damage
						if (terrain.map[indexUp] == Terrain::eType::air ||
							terrain.map[indexUp] == Terrain::eType::water)
						{
							// talk to the ai
							if(objPooper.ai.bFoundPipe == false)
							{
								objPooper.ai.bFoundPipe = true;
								objPooper.ai.targetx = x;
								objPooper.ai.pipePos = x;
								
								// need to change state here perhaps..

								//objPooper.ai.targety = y???
							}
							// end talk

							//static int n = 0;
							//n++;
							debreSpawnCounter++;
							if (debreSpawnCounter < 20)
							{   // exposed pipe
								for (int i = 0; i < 10; i++)
								{   // type 2 single yellow pixel particle
									debreIndexCounter = AdvanceDebreIndexConter();
									float rndX = 0.025f * ((((float)rand() / (RAND_MAX))*2.f) - 1.f); // (-1 to +1)*scale
									float rndY = -0.85f * ((float)rand() / (RAND_MAX)); // 0 to -scale w/intensity modifier
									debreObjs[debreIndexCounter].pos = vec2(x, tty);// obj->pos;
									debreObjs[debreIndexCounter].vel = { rndX, rndY };
									debreObjs[debreIndexCounter].nextVel = { rndX, rndY };
									debreObjs[debreIndexCounter].bDead = false;
									debreObjs[debreIndexCounter].bExplode = false;
									debreObjs[debreIndexCounter].bHitThisFrame = false;
									debreObjs[debreIndexCounter].id = i;
									debreObjs[debreIndexCounter].mass = 0.002f;
									debreObjs[debreIndexCounter].radius = 0.003f;
									debreObjs[debreIndexCounter].nBounceCount = 0;
									debreObjs[debreIndexCounter].nBounceBeforDeath = 1;
									debreObjs[debreIndexCounter].type = 0; // single pixel debre
									debreObjs[debreIndexCounter].color = olc::Pixel(180, 180, 255, 255);
								}
							}
						}
					}
				}
			}
			if (tty == 0) // top row tested. reset to bottom to restart scan.
				tty = terrain.nMapHeight - 1;
		}
	}
#pragma endregion 

	int sw = e->ScreenWidth();
	int sh = e->ScreenHeight();
	int cx = (int)camera.posx;
	int cy = (int)camera.posy;
	int r = 70;
	int g = 70;
	int b = 255;
	float s = 255.f / (e->ScreenHeight()); // background gradient

#pragma region draw_landscape

	for (int x = 0; x < e->ScreenWidth(); x++)
	{
		float step = 255.f; // background sky gradient
		int tStep = 165;    // terrain gradient
		for (int y = 0; y < e->ScreenHeight(); y++)
		{
			switch (terrain.map[(y + cy)*terrain.nMapWidth + (x + cx)])
			{
			case Terrain::eType::air:
			{   // sky
				int c = (uint8_t)step - 127;
				if (c < 0) c = 0;
				e->Draw(x, y, olc::Pixel(r, g, c, 255));
				break;
			}
			case Terrain::eType::land: // ground
				e->Draw(x, y, olc::Pixel(uint8_t((80 - (step / 2)) * 0.65f), uint8_t(tStep*0.85f), 40, 255));
				tStep -= y / 305;// ((terrain.nMapHeight - y) / 55);
				if (tStep < 0) tStep = 0;
				break;
			case Terrain::eType::water: // water
				e->Draw(x, y, olc::Pixel(220, 220, 255, 255));
				break;
			case Terrain::eType::goal:
				e->Draw(x, y, olc::BLACK); // pixel perfect collision with balls
				break;
			//case Terrain::eType::pipe: // pipe
			//	e->Draw(x, y, olc::Pixel(0, 0, 255, 255)); // debug drawing 
			//	break;
			}
			step -= s;
		}
	}

#pragma endregion

#pragma region physics_objects 
	// draw physics lists (the balls)
	for (auto &obj : physicsObjs)
	{
		if (!obj->bDead)
			obj->Draw(e, (float)cx, (float)cy);
		// reset debug color
		obj->color = olc::RED;
	}

	// draw debre list (the particle system)
	int countDebre = 0;
	for (int i = 0; i < 512; i++)
	{
		if (!debreObjs[i].bDead)
		{
			if (debreObjs[i].pos.y < 0) { debreObjs[i].bDead = true; }
			debreObjs[i].Draw(e, float(cx), (float)cy);
			countDebre++;
		}
	}
#pragma endregion

	// the pooper and it's pin -------------------------------------------------------
	objPooper.draw(e, cx, cy);
	objPooper.objGoalPin.Draw(e, (float)cx, (float)cy);

	// the bumper(s) -----------------------------------------------------------------
	for (auto &obj : objBumpers)
	{
		obj->draw(e, cx, cy);
	}

	// goal bucket target animation ---------------------------------------------------
	vec2 objGpos = {objPlayerGoalPosition[(pipes.pipeLayoutIndex*2)], objPlayerGoalPosition[(pipes.pipeLayoutIndex*2+1)]};
	e->DrawCircle(objGpos.x - cx + 21, objGpos.y - cy + 3, 20 * (1.0f - goalAnimTick), olc::BLUE);
	e->DrawCircle(objGpos.x - cx + 21, objGpos.y - cy + 3, 15 * (1.0f - goalAnimTick), olc::YELLOW);
	e->DrawCircle(objGpos.x - cx + 21, objGpos.y - cy + 3, 10 * (1.0f - goalAnimTick), olc::RED);


	// ********************************************************************************
	// ******************** Begin Painters / Transparency Drawing *********************
	e->SetPixelMode(olc::Pixel::ALPHA);
	// the power ups ----------------------------------------------------------
	powerups.draw(e, cx, cy);
	
	// the hose bit -----------------------------------------------------------
	objHoseBit.update(&terrain);
	objHoseBit.draw(camera.posx, camera.posy, false);
	
	// the player -------------------------------------------------------------
	player.Draw(e, float(cx), float(cy));
	if (player.bFootstep) {
		manager->playSound(player.iSnd[player.iSndCurrent], eSoundTypes::bell, 0.0, 0.05);
		player.iSndCurrent++; if (player.iSndCurrent == 12) player.iSndCurrent = 0;
		player.bFootstep = false;
	}

	// texture pipes ----------------------------------------------------------
	pipes.draw(e, cx, cy);
	
	// the win bucket --------------------------------------------------------
	e->DrawSprite(objGpos.x-cx, objGpos.y-cy, &objPlayerGoal);
	e->SetPixelMode(olc::Pixel::NORMAL);
	// *********************************************************************************
	// ********************** End Painters / Transparency Drawing **********************   	

	

	// loose game event -------------------------------------------------------
	int index = objPooper.objGoalPin.pos.y*terrain.nMapWidth + objPooper.objGoalPin.pos.x;
	if (terrain.map[index] == Terrain::eType::pipe)
	{
		// start particle spray
		objPooper.objGoalPin.bStable = true;
		manager->handleEvents(Core::eStateAction::loose);
	}

	e->DrawString(15, 385, getLevelTimeString());

	/* ai debug
	char buf[64];
	sprintf_s(buf, sizeof(buf), "T (%d, %d)", objPooper.ai.targetx, objPooper.ai.targety);
	e->DrawString(10, 20, buf);
	sprintf_s(buf, sizeof(buf), "P (%d, %d)", objPooper.ai.posx, objPooper.ai.posy);
	e->DrawString(10, 35, buf);
	_itoa_s(objPooper.ai.m_machine->getCurrentState()->m_type, buf, 10);
	std::string st = "typ ";
	e->DrawString(10, 50, st + buf);
	objPooper.ai.bFoundPipe ? sprintf_s(buf, sizeof(buf), "(X) Found") : sprintf_s(buf, sizeof(buf), "( ) Found");
	e->DrawString(10, 65, buf);
	*/

	/*/ debug information
	char buf[16];
	_itoa_s(int(objPooper.objGoalPin.pos.x), buf, 10);
	e->DrawString(10, 10, std::string(buf));
	_itoa_s(int(objPooper.objGoalPin.pos.y), buf, 10);
	e->DrawString(50, 10, std::string(buf));
	_itoa_s(cx, buf, 10);
	e->DrawString(10, 30, std::string(buf));
	_itoa_s(cy, buf, 10);
	e->DrawString(50, 30, std::string(buf));
	_itoa_s(countDebre, buf, 10);
	e->DrawString(20, 40, buf);*/

	//if (isDebug_player /*F1*/)
	//{
	//}

	/*{
		e->SetPixelMode(olc::Pixel::ALPHA);
		olc::Pixel bkgColor(20, 20, 10, 156);
		int pdx = 5;
		int pdy = 5;
		e->FillRect(7 + pdx, pdy, 206, 280, bkgColor);
		e->FillRect(pdx, 7 + pdy, 220, 267, bkgColor);
		e->FillTriangle(213 + pdx, pdy+1, 213 + pdx, 10 + pdy, 219 + pdx, 10 + pdy, bkgColor);       //  |_
		e->FillTriangle(10 + pdx, pdy+1, 10 + pdx, 10 + pdy, pdx+1, 10 + pdy, bkgColor);             //  _|
		e->FillTriangle(213 + pdx, 279 + pdy, 213 + pdx, 273 + pdy, 219 + pdx, 273 + pdy, bkgColor); // |`
		e->FillTriangle(pdx+1, 273 + pdy, 10 + pdx, 273 + pdy, 10 + pdx, 279 + pdy, bkgColor);       // `|
		e->DrawRect(6 + pdx, 6 + pdy, 208, 269, olc::DARK_BLUE);
		e->SetPixelMode(olc::Pixel::NORMAL);
		pdx += 15; pdy += 15;
		e->DrawString(pdx, pdy, player.isGrounded ? "(X)isGrounded" : "( )isGrounded"); pdy += 12;
		e->DrawString(pdx, pdy, player.isJumping ? "(X)isJumping" : "( )isJumping"); pdy += 12;
		e->DrawString(pdx, pdy, (player.animState==eAnimState::block) ? "(X)isBlocking" : "( )isBlocking");
	}*/

	/*int px = 5;
	int py = 5;
	char buf[32] = "";
	e->DrawString(px, py, "Level ");
	_itoa_s(thisLevel, buf, 10);
	e->DrawString(px + 50, py, buf);*/
}
