// file : gameState.h

#ifndef _engine_core_states_game_h_
#define _engine_core_states_game_h_

#include "base_state.h"

#include "../TerrainMap.h"
#include "../Camera.h"
#include "../PhysicsObject.h"
#include "../Actors/Player.h"
#include "../SFX/Water.h"

#define OLC_PGE_GRAPHICS2D
#include <olcPGEX_Graphics2D.h>

#include "../Actors/HoseBit.h"
#include "../Actors/SuperPooper.h"

#include "../Actors/Bumper.h"

// my todo list (19-05-06) added sound
//     returned (19-05-13) added game logic framework
//     returned (19-05-23) added pipe layouts 
//     returned (19-05-24) added ai state machine

// todo: 
//  credits out
//  scoring???
//
//  build 3 puzzles (&powerup) and a win screen???
//


namespace Core {
	namespace States {

		class Game : public Base_state {
		public:
			Game(void* _manager, vec2 _clientSize) : Base_state(_manager, "game_state", _clientSize) { initialize(); }

			void initialize();
			void shutdown();
			void update(float _deltaTime);
			void draw();

			void pause();
			void resume();
			void reset(int level);
			void handleEvents();

			char* getLevelTimeString();

			// unused.............todo.............
			char* getWinnerName();
			void drawFloorDivider(); // not used...
			void drawReadyServe();

			// debug screens
			bool isDebug_player = false;

		private:
			int nLevelNum = 1;
			bool isLevelCompleted = false;
			double playerGameTime = 0;

			Camera camera;
			Player player;
			Terrain terrain;

			HoseBit objHoseBit;
			SuperPooper objPooper;
			std::vector<std::unique_ptr<Bumper>> objBumpers;
		
			std::list<std::unique_ptr<PhysicsObject>> physicsObjs;
			

			Water waterEmitter = { {350.f, 0.f}, {0.f, 0.f}, true };
			Debre debreObjs[512]; // fixed array of aux/minor effect objects (next empty rotatated)


			olc::Sprite objPlayerGoal;
			float objPlayerGoalPosition[12] = { 975, 220, 1015, 230, 1054, 220, 958, 256,  815, 207,  878, 248 };
			float goalAnimTick = 0.f;
		};
	}
}

#endif

