#include "olcPixelGameEngine.h"
#include "TerrainMap.h"

// one dimensional perlin noise --------------------------
// https://youtu.be/6-0UaeJBumA
void PerlinNoise1D(int nCount, float *fSeed, int nOctaves, float fBias, float *fOutput)
{
	for (int x = 0; x < nCount; x++)
	{
		float fNoise = 0.f;
		float fScaleAcc = 0.f;
		float fScale = 1.f;

		for (int o = 0; o < nOctaves; o++)
		{
			int nPitch = nCount >> o; // halved
			int nSample1 = (x / nPitch) * nPitch;
			int nSample2 = (nSample1 + nPitch) % nCount; // wrap
			float fBlend = (float)(x - nSample1) / (float)nPitch;
			float fSample = (1.f - fBlend) * fSeed[nSample1] + fBlend * fSeed[nSample2];
			fScaleAcc += fScale;
			fNoise += fSample * fScale;
			fScale = fScale / fBias;
		}

		// Scale the seed range
		fOutput[x] = fNoise / fScaleAcc;
	}
}

// https://www.youtube.com/watch?v=EHlaJvQpW3U&t=76s
int seedIterator = -1;
// 124 kill level...interesting back end over flow to front on trigger
// 118 121 high
// 117, 14 120, 16  (low to high) but flat
// 7, 8, 10, 11 low
// 13 really low 119
const int numSeeds = 6; // 6 unique terrain configurations
int seeds[numSeeds] = { 200, 120, 14, 117, 302, 304 };


void Terrain::Reset()
{
	seedIterator = -1;
	//CreateMap();
}

bool Terrain::CreateMap()
{
	if (map != nullptr)
		delete map;

	int numElements = nMapWidth * nMapHeight;
	map = new unsigned char[numElements];
	memset(map, 0, numElements * sizeof(unsigned char));

	float *fSurface = new float[nMapWidth];    // height values
	float *fNoiseSeed = new float[nMapWidth];  // noise values
	// generate some 1D noise
	seedIterator++; if (seedIterator == numSeeds) seedIterator = 0; // increment and restart
	srand(seeds[seedIterator]); // the unique-afier .....
	for (int i = 0; i < nMapWidth; i++)
	{
		fNoiseSeed[i] = (float)rand() / (float)RAND_MAX;
	}

	fNoiseSeed[0] = 0.5f; // first and last(scanline)(y axis) normalied location (start center_y)
	PerlinNoise1D(nMapWidth, fNoiseSeed, 8, 2.f, fSurface);
	// generate the land
	for (int x = 0; x < nMapWidth; x++)
	{
		for (int y = 0; y < nMapHeight; y++)
		{
			if (y >= fSurface[x] * nMapHeight)
				map[y * nMapWidth + x] = 1;
			else
				map[y * nMapWidth + x] = 0;
		}
	}

	delete[] fSurface;
	delete[] fNoiseSeed;

	return true;
}

void Terrain::Update(float fElapsedTime)
{

}

void Terrain::Carve(vec2 pos, float radius, float strength)
{
	// todo : check if we ever index out of range

	if (radius < 1.f) radius = 1.f;
	radius *= 2.25f * strength;
	pos.x -= 1;

	float distSquared = radius * radius;
	int startX = int(pos.x - radius); // if startX < 0 || startX > nMapWidth
	int startY = int(pos.y - radius); // ------same--------
	int endX = int(pos.x + radius);    // different...split into two tests ------------ TODO ---------------
	int endY = int(pos.y + radius);
	int stride = int(radius + radius); //-1 ?????
	bool functionKiller = false;

	float vx, vy, d2;
	for (int i = startX; i < endX; i++)
	{
		if (i > nMapWidth) break;
		for (int j = startY; j < endY; j++)
		{
			if (j > nMapHeight) break;
			if (map[(j*nMapWidth) + i] == eType::pipe) break; // stop destruction at pipe for this vertical slice

			vx = (i < pos.x) ? pos.x - i : i - pos.x; 
			vy = (j < pos.y) ? pos.y - j : j - pos.y; 
			d2 = (vx * vx) + (vy * vy);
			if (d2 < distSquared)
			{
				if (map[(j*nMapWidth) + i] == eType::land) map[(j*nMapWidth) + i] = eType::air;
			}
		}
	}
}

void Terrain::Buldge(vec2 pos, float radius, float strength)
{
}

